#include <linux/module.h>	
#include <linux/kernel.h>	/* KERN_INFO */
#include <linux/init.h>		/* macros */

#include <linux/proc_fs.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#include <linux/tty.h>

#define DRIVER_AUTHOR "Grande Puffo"
#define DRIVER_DESC   "Sample ProcEntry"

/* Declare code as GPL. */
MODULE_LICENSE("GPL");

MODULE_AUTHOR(DRIVER_AUTHOR);	
MODULE_DESCRIPTION(DRIVER_DESC);


static struct proc_dir_entry *proc_entry;
static struct proc_dir_entry *dir_entry;
static char *buffer;

#define DIRNAME "SampleDir"
#define FILENAME "SampleEntry"

ssize_t proc_write(struct file *filp, const char *buff,
	unsigned long len, void *data)
{
	if (len>=PAGE_SIZE) {
		len=PAGE_SIZE-1;
	}
	if (copy_from_user(buffer,buff,len)) {
		return -EFAULT;
	}
	buffer[len]='\0';
	return len;
}
ssize_t proc_read( char *page, char **start, off_t off,
	int count, int *eof, void *data)
{
	int len;

	if (off>0) {
		*eof=1;
		return 0;
	}
	len=sprintf(page,"%s",buffer);
	return len;
}


int init_module(void)
{
	if (!(dir_entry=proc_mkdir(DIRNAME,NULL))) {
		printk(KERN_INFO "mkdir failed");
		return(-1);
	}
	if (!(proc_entry=create_proc_entry(FILENAME,0644,dir_entry))) {
		printk(KERN_INFO "create_proc_entry failed");
		return(-1);
	}
	proc_entry->read_proc=proc_read;
	proc_entry->write_proc=proc_write;
	buffer = (char *)vmalloc(PAGE_SIZE);
	if (!buffer) {
		return -ENOMEM;
	}
	buffer[0]='\0';
	printk(KERN_INFO "Module Loaded\n");
	return 0 ;
}

void cleanup_module(void)
{
	vfree(buffer);
	remove_proc_entry(FILENAME,dir_entry);
	remove_proc_entry(DIRNAME,NULL);
	printk(KERN_INFO "Module Removed\n");
}
